@@ -48,22 +48,22 @@ from .group import GroupCommandProcessor
 class QueryCommandProcessor(GroupCommandProcessor):
 
     # Override
-    def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
+    async def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
         assert isinstance(content, QueryCommand), 'query command error: %s' % content
 
         # 0. check command
-        group, errors = self._check_expired(content=content, r_msg=r_msg)
+        group, errors = await self._check_expired(content=content, r_msg=r_msg)
         if group is None:
             # ignore expired command
             return errors
 
         # 1. check group
-        owner, members, errors = self._check_group_members(content=content, r_msg=r_msg)
+        owner, members, errors = await self._check_group_members(content=content, r_msg=r_msg)
         if owner is None or len(members) == 0:
             return errors
 
         sender = r_msg.sender
-        bots = self._assistants(group=group)
+        bots = await self._assistants(group=group)
         is_member = sender in members
         is_bot = sender in bots
         can_query = is_member or is_bot
@@ -72,9 +72,9 @@ class QueryCommandProcessor(GroupCommandProcessor):
         if not can_query:
             text = 'Permission denied.'
             return self._respond_receipt(text=text, content=content, envelope=r_msg.envelope, extra={
-                'template': 'Not allowed to query members of group: ${ID}',
+                'template': 'Not allowed to query members of group: ${gid}',
                 'replacements': {
-                    'ID': str(group),
+                    'gid': str(group),
                 }
             })
 
@@ -82,23 +82,23 @@ class QueryCommandProcessor(GroupCommandProcessor):
         query_time = content.last_time
         if query_time is not None:
             # check last group history time
-            archivist = self.facebook.archivist
-            last_time = archivist.get_last_group_history_time(group=group)
+            checker = self.facebook.checker
+            last_time = await checker.get_last_group_history_time(group=group)
             if last_time is None:
                 self.error(msg='group history error: %s' % group)
             elif not last_time.after(query_time):
                 # group history not updated
                 text = 'Group history not updated.'
                 return self._respond_receipt(text=text, content=content, envelope=r_msg.envelope, extra={
-                    'template': 'Group history not updated: ${ID}, last time: ${time}',
+                    'template': 'Group history not updated: ${gid}, last time: ${time}',
                     'replacements': {
-                        'ID': str(group),
+                        'gid': str(group),
                         'time': last_time.timestamp,
                     }
                 })
 
         # 3. send newest group history commands
-        ok = self.send_group_histories(group=group, receiver=sender)
+        ok = await self.send_group_histories(group=group, receiver=sender)
         assert ok, 'failed to send history for group: %s => %s' % (group, sender)
 
         # no need to response this group command
