@@ -44,7 +44,7 @@ from ...common import HandshakeCommand
 class HandshakeCommandProcessor(BaseCommandProcessor, Logging):
 
     # Override
-    def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
+    async def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
         assert isinstance(content, HandshakeCommand), 'handshake command error: %s' % content
         messenger = get_client_messenger(cpu=self)
         client_session = get_client_session(messenger=messenger)
@@ -60,7 +60,7 @@ class HandshakeCommandProcessor(BaseCommandProcessor, Logging):
         # handle handshake command with title & session key
         title = content.title
         new_sess_key = content.session
-        old_sess_key = client_session.key
+        old_sess_key = client_session.session_key
         assert new_sess_key is not None, 'new session key should not be empty: %s' % content
         if 'DIM?' == title:
             # S -> C: station ask client to handshake again
@@ -68,17 +68,17 @@ class HandshakeCommandProcessor(BaseCommandProcessor, Logging):
             # clear client session key while handshake again
             if old_sess_key is None:
                 # first handshake response with new session key,
-                messenger.handshake(session_key=new_sess_key)
+                await messenger.handshake(session_key=new_sess_key)
             elif old_sess_key == new_sess_key:
                 # duplicated handshake response?
                 # or session expired and the station ask to handshake again?
                 self.warning(msg='session key already set: %s => %s' % (old_sess_key, new_sess_key))
-                messenger.handshake(session_key=new_sess_key)
+                await messenger.handshake(session_key=new_sess_key)
             else:
                 # connection changed?
                 self.error(msg='session key from %s not match: %s => %s' % (sender, old_sess_key, new_sess_key))
                 # erase session key to handshake again
-                client_session.key = None
+                client_session.session_key = None
         elif 'DIM!' == title:
             # S -> C: handshake accepted by station
             self.info(msg='handshake success: %s, local: %s' % (station.identifier, client_session.identifier))
@@ -86,7 +86,7 @@ class HandshakeCommandProcessor(BaseCommandProcessor, Logging):
             if old_sess_key is None:
                 # normal handshake response,
                 # update session key to change state to 'running'
-                client_session.key = new_sess_key
+                client_session.session_key = new_sess_key
             elif old_sess_key == new_sess_key:
                 # duplicated handshake response?
                 pass
@@ -94,7 +94,7 @@ class HandshakeCommandProcessor(BaseCommandProcessor, Logging):
                 # FIXME: handshake error
                 self.error(msg='session key from %s not match: %s => %s' % (sender, old_sess_key, new_sess_key))
                 # erase session key to handshake again
-                client_session.key = None
+                client_session.session_key = None
         else:
             # C -> S: Hello world!
             self.error(msg='[Error] handshake command from %s: %s' % (sender, content))
