@@ -28,41 +28,48 @@
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 """
 
-import time
-from typing import List
+from typing import Optional, List
 
 from dimsdk import DateTime
 from dimsdk import EntityType
 from dimsdk import ReliableMessage
 from dimsdk import Content, TextContent
 from dimsdk import ReceiptCommand
+from dimsdk import Facebook, Messenger
 from dimsdk import ContentProcessorCreator
 
 from ..common import HandshakeCommand
-from ..common import CommonMessenger
+from ..common import CommonFacebook, CommonMessenger
 from ..common import CommonMessageProcessor
 
-from .cpu import ClientContentProcessorCreator
-
-from .archivist import ClientArchivist
-
 
 class ClientMessageProcessor(CommonMessageProcessor):
 
     @property
-    def messenger(self) -> CommonMessenger:
+    def facebook(self) -> Optional[CommonFacebook]:
+        barrack = super().facebook
+        assert isinstance(barrack, CommonFacebook), 'facebook error: %s' % barrack
+        return barrack
+
+    @property
+    def messenger(self) -> Optional[CommonMessenger]:
         transceiver = super().messenger
         assert isinstance(transceiver, CommonMessenger), 'messenger error: %s' % transceiver
         return transceiver
 
+    # Override
+    def _create_creator(self, facebook: Facebook, messenger: Messenger) -> ContentProcessorCreator:
+        from .cpu import ClientContentProcessorCreator
+        return ClientContentProcessorCreator(facebook=facebook, messenger=messenger)
+
     # private
-    def _check_group_times(self, content: Content, r_msg: ReliableMessage) -> bool:
+    async def _check_group_times(self, content: Content, r_msg: ReliableMessage) -> bool:
         group = content.group
         if group is None:
             return False
-        facebook = self.facebook
-        archivist = facebook.archivist
-        assert isinstance(archivist, ClientArchivist), 'client archivist error: %s' % archivist
+        else:
+            facebook = self.facebook
+            checker = facebook.checker
         now = DateTime.now()
         doc_updated = False
         mem_updated = False
@@ -72,31 +79,32 @@ class ClientMessageProcessor(CommonMessageProcessor):
             if last_doc_time.after(now):
                 # calibrate the clock
                 last_doc_time = now
-            doc_updated = archivist.set_last_document_time(identifier=group, last_time=last_doc_time)
+            doc_updated = checker.set_last_document_time(identifier=group, now=last_doc_time)
             # check whether needs update
             if doc_updated:
                 self.info(msg='checking for new bulletin: %s' % group)
-                facebook.documents(identifier=group)
+                await facebook.get_documents(identifier=group)
         # check group history time
         last_his_time = r_msg.get_datetime(key='GHT', default=None)
         if last_his_time is not None:
             if last_his_time.after(now):
                 # calibrate the clock
                 last_his_time = now
-            mem_updated = archivist.set_last_group_history_time(group=group, last_time=last_his_time)
+            mem_updated = checker.set_last_group_history_time(identifier=group, now=last_his_time)
             # check whether needs update
             if mem_updated:
-                archivist.set_last_active_member(member=r_msg.sender, group=group)
+                checker.set_last_active_member(member=r_msg.sender, group=group)
                 self.info(msg='checking for group members: %s' % group)
-                facebook.members(identifier=group)
+                await facebook.get_members(identifier=group)
+        # OK
         return doc_updated or mem_updated
 
     # Override
-    def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
-        responses = super().process_content(content=content, r_msg=r_msg)
+    async def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
+        responses = await super().process_content(content=content, r_msg=r_msg)
         # check group document & history times from the message
         # to make sure the group info synchronized
-        self._check_group_times(content=content, r_msg=r_msg)
+        await self._check_group_times(content=content, r_msg=r_msg)
         # check responses
         if len(responses) == 0:
             # respond nothing
@@ -106,34 +114,28 @@ class ClientMessageProcessor(CommonMessageProcessor):
             return responses
         sender = r_msg.sender
         receiver = r_msg.receiver
-        user = self.facebook.select_user(receiver=receiver)
-        if user is None:
+        me = await self.facebook.select_local_user(receiver=receiver)
+        if me is None:
             # assert False, 'receiver error: %s' % receiver
             return responses
-        receiver = user.identifier
         messenger = self.messenger
         # check responses
+        from_bots = sender.type == EntityType.STATION or sender.type == EntityType.BOT
         for res in responses:
             if res is None:
                 # should not happen
                 continue
             elif isinstance(res, ReceiptCommand):
-                if sender.type == EntityType.STATION or sender.type == EntityType.BOT:
+                if from_bots:
                     # no need to respond receipt to station
-                    when = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(r_msg.time))
-                    self.info(msg='drop receipt responding to %s, origin msg time=[%s]' % (sender, when))
+                    self.info(msg='drop receipt to %s, origin msg time=[%s]' % (sender, r_msg.time))
                     continue
             elif isinstance(res, TextContent):
-                if sender.type == EntityType.STATION or sender.type == EntityType.BOT:
+                if from_bots:
                     # no need to respond text message to station
-                    when = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(r_msg.time))
-                    self.info(msg='drop text responding to %s, origin time=[%s], text=%s' % (sender, when, res.text))
+                    self.info(msg='drop text to %s, origin time=[%s], text=%s' % (sender, r_msg.time, res.text))
                     continue
             # normal response
-            messenger.send_content(sender=receiver, receiver=sender, content=res)
+            await messenger.send_content(sender=me, receiver=sender, content=res, priority=1)
         # DON'T respond to station directly
         return []
-
-    # Override
-    def _create_creator(self) -> ContentProcessorCreator:
-        return ClientContentProcessorCreator(facebook=self.facebook, messenger=self.messenger)
