@@ -34,39 +34,45 @@ from .dbi import *
 
 from .anonymous import Anonymous
 from .register import Register
+from .ans import AddressNameService
 from .ans import AddressNameServer, ANSFactory
 
+from .checker import EntityChecker
 from .archivist import CommonArchivist
 from .facebook import CommonFacebook
 from .messenger import CommonMessenger
 from .packer import CommonMessagePacker
 from .processer import CommonMessageProcessor
+from .queue import SuspendedMessageQueue
 from .session import Transmitter, Session
 
-from .compat import register_compatible_factories
-
-
-register_all_factories()
-register_plugins()
-register_compatible_factories()
-
 
 __all__ = [
 
+    'MetaVersion',
+    'Password',
+    'BroadcastUtils',
+
     #
     #   protocol
     #
+
+    'AnsCommand',
+
     'HandshakeCommand', 'HandshakeState',
     'LoginCommand',
-    'ReportCommand',
-    'AnsCommand',
 
     'BlockCommand',
     'MuteCommand',
 
+    'ReportCommand',
+
+    'CustomizedContent', 'AppCustomizedContent',
+
     #
     #   Database Interface
     #
+
     'PrivateKeyDBI', 'MetaDBI', 'DocumentDBI',
     'UserDBI', 'ContactDBI', 'GroupDBI', 'GroupHistoryDBI',
     'AccountDBI',
@@ -74,22 +80,32 @@ __all__ = [
     'ReliableMessageDBI', 'CipherKeyDBI', 'GroupKeysDBI',
     'MessageDBI',
 
-    'LoginDBI', 'ProviderDBI', 'StationDBI',
+    'ProviderDBI', 'StationDBI', 'LoginDBI',
     'SessionDBI',
+
     'ProviderInfo', 'StationInfo',
 
     #
     #   common
     #
-    'Anonymous', 'Register',
-    'AddressNameServer', 'ANSFactory',
 
+    'Anonymous',
+    'AddressNameService',
+    'AddressNameServer',
+    'ANSFactory',
+
+    'EntityChecker',
     'CommonArchivist',
     'CommonFacebook',
+
     'CommonMessenger',
     'CommonMessagePacker',
     'CommonMessageProcessor',
+    'SuspendedMessageQueue',
+
     'Transmitter',
     'Session',
 
+    'Register',
+
 ]
