@@ -25,6 +25,7 @@
 
 from abc import ABC, abstractmethod
 from typing import Optional, Any, Dict, List, Tuple
+from typing import Iterable
 
 from dimsdk import ID, Identifier, EVERYWHERE
 from dimsdk import Station
@@ -33,21 +34,6 @@ from dimsdk import ReliableMessage
 from ..protocol import LoginCommand
 
 
-class LoginDBI(ABC):
-    """ Login Command Table """
-
-    #
-    #   login command message
-    #
-    @abstractmethod
-    def login_command_message(self, user: ID) -> Tuple[Optional[LoginCommand], Optional[ReliableMessage]]:
-        raise NotImplemented
-
-    @abstractmethod
-    def save_login_command_message(self, user: ID, content: LoginCommand, msg: ReliableMessage) -> bool:
-        raise NotImplemented
-
-
 #
 #   Service Provider
 #
@@ -56,30 +42,32 @@ class LoginDBI(ABC):
 class ProviderInfo:
 
     # default service provider
-    GSP = Identifier(identifier='gsp@everywhere', name='gsp', address=EVERYWHERE)
+    GSP = Identifier.new(name='gsp', address=EVERYWHERE)
 
     def __init__(self, identifier: ID, chosen: int):
         super().__init__()
         self.identifier = identifier
         self.chosen = chosen
 
+    # Override
     def __str__(self) -> str:
         clazz = self.__class__.__name__
         return '<%s ID="%s" chosen=%d />' % (clazz, self.identifier, self.chosen)
 
+    # Override
     def __repr__(self) -> str:
         clazz = self.__class__.__name__
         return '<%s ID="%s" chosen=%d />' % (clazz, self.identifier, self.chosen)
 
     @classmethod
-    def convert(cls, array: List[Dict[str, Any]]):  # -> List[ProviderInfo]:
+    def convert(cls, array: Iterable[Dict[str, Any]]):  # -> List[ProviderInfo]:
         gf = ProviderFactoryManager.general_factory
         return gf.convert_providers(array=array)
 
     @classmethod
-    def revert(cls, array) -> List[Dict[str, Any]]:
+    def revert(cls, providers) -> List[Dict[str, Any]]:
         gf = ProviderFactoryManager.general_factory
-        return gf.revert_providers(array=array)
+        return gf.revert_providers(providers=providers)
 
 
 class StationInfo:
@@ -94,25 +82,27 @@ class StationInfo:
         self.provider = provider
         self.chosen = chosen
 
+    # Override
     def __str__(self) -> str:
         clazz = self.__class__.__name__
         return '<%s host="%s" port=%d ID="%s" SP="%s" chosen=%d />' % (clazz, self.host, self.port, self.identifier,
                                                                        self.provider, self.chosen)
 
+    # Override
     def __repr__(self) -> str:
         clazz = self.__class__.__name__
         return '<%s host="%s" port=%d ID="%s" SP="%s" chosen=%d />' % (clazz, self.host, self.port, self.identifier,
                                                                        self.provider, self.chosen)
 
     @classmethod
-    def convert(cls, array: List[Dict[str, Any]]):  # -> List[StationInfo]:
+    def convert(cls, array: Iterable[Dict[str, Any]]):  # -> List[StationInfo]:
         gf = ProviderFactoryManager.general_factory
         return gf.convert_stations(array=array)
 
     @classmethod
-    def revert(cls, array) -> List[Dict[str, Any]]:
+    def revert(cls, stations) -> List[Dict[str, Any]]:
         gf = ProviderFactoryManager.general_factory
-        return gf.revert_stations(array=array)
+        return gf.revert_stations(stations=stations)
 
 
 class ProviderGeneralFactory:
@@ -121,10 +111,13 @@ class ProviderGeneralFactory:
         super().__init__()
 
     # noinspection PyMethodMayBeStatic
-    def convert_providers(self, array: List[Dict[str, Any]]) -> List[ProviderInfo]:
+    def convert_providers(self, array: Iterable[Dict[str, Any]]) -> List[ProviderInfo]:
         providers = []
         for item in array:
-            identifier = ID.parse(identifier=item.get('ID'))
+            did = item.get('did')
+            if did is None:
+                did = item.get('ID')
+            identifier = ID.parse(identifier=did)
             chosen = item.get('chosen')
             if identifier is None:
                 # provider ID error
@@ -134,20 +127,24 @@ class ProviderGeneralFactory:
         return providers
 
     # noinspection PyMethodMayBeStatic
-    def revert_providers(self, array: List[ProviderInfo]) -> List[Dict[str, Any]]:
-        providers = []
-        for item in array:
-            providers.append({
+    def revert_providers(self, providers: Iterable[ProviderInfo]) -> List[Dict[str, Any]]:
+        array = []
+        for item in providers:
+            array.append({
                 'ID': str(item.identifier),
+                'did': str(item.identifier),
                 'chosen': item.chosen,
             })
-        return providers
+        return array
 
     # noinspection PyMethodMayBeStatic
-    def convert_stations(self, array: List[Dict[str, Any]]) -> List[StationInfo]:
+    def convert_stations(self, array: Iterable[Dict[str, Any]]) -> List[StationInfo]:
         stations = []
         for item in array:
-            identifier = ID.parse(identifier=item.get('ID'))
+            did = item.get('did')
+            if did is None:
+                did = item.get('ID')
+            identifier = ID.parse(identifier=did)
             host = item.get('host')
             port = item.get('port')
             provider = ID.parse(identifier=item.get('provider'))
@@ -162,17 +159,18 @@ class ProviderGeneralFactory:
         return stations
 
     # noinspection PyMethodMayBeStatic
-    def revert_stations(self, array: List[StationInfo]) -> List[Dict[str, Any]]:
-        stations = []
-        for item in array:
-            stations.append({
+    def revert_stations(self, stations: Iterable[StationInfo]) -> List[Dict[str, Any]]:
+        array = []
+        for item in stations:
+            array.append({
                 'ID': str(item.identifier),
+                'did': str(item.identifier),
                 'host': item.host,
                 'port': item.port,
                 'provider': str(item.provider),
                 'chosen': item.chosen,
             })
-        return stations
+        return array
 
 
 class ProviderFactoryManager:
@@ -184,48 +182,65 @@ class ProviderDBI(ABC):
     """ Provider Stations Table """
 
     @abstractmethod
-    def all_providers(self) -> List[ProviderInfo]:
+    async def all_providers(self) -> List[ProviderInfo]:
         """ get list of (SP_ID, chosen) """
         raise NotImplemented
 
     @abstractmethod
-    def add_provider(self, identifier: ID, chosen: int = 0) -> bool:
+    async def add_provider(self, identifier: ID, chosen: int = 0) -> bool:
         raise NotImplemented
 
     @abstractmethod
-    def update_provider(self, identifier: ID, chosen: int) -> bool:
+    async def update_provider(self, identifier: ID, chosen: int) -> bool:
         raise NotImplemented
 
     @abstractmethod
-    def remove_provider(self, identifier: ID) -> bool:
+    async def remove_provider(self, identifier: ID) -> bool:
         raise NotImplemented
 
 
 class StationDBI(ABC):
 
     @abstractmethod
-    def all_stations(self, provider: ID) -> List[StationInfo]:
+    async def all_stations(self, provider: ID) -> List[StationInfo]:
         """ get list of (host, port, SP_ID, chosen) """
         raise NotImplemented
 
     @abstractmethod
-    def add_station(self, identifier: Optional[ID], host: str, port: int, provider: ID, chosen: int = 0) -> bool:
+    async def add_station(self, identifier: Optional[ID], host: str, port: int, provider: ID,
+                          chosen: int = 0) -> bool:
         raise NotImplemented
 
     @abstractmethod
-    def update_station(self, identifier: Optional[ID], host: str, port: int, provider: ID, chosen: int = None) -> bool:
+    async def update_station(self, identifier: Optional[ID], host: str, port: int, provider: ID,
+                             chosen: int = None) -> bool:
         raise NotImplemented
 
     @abstractmethod
-    def remove_station(self, host: str, port: int, provider: ID) -> bool:
+    async def remove_station(self, host: str, port: int, provider: ID) -> bool:
         raise NotImplemented
 
     @abstractmethod
-    def remove_stations(self, provider: ID) -> bool:
+    async def remove_stations(self, provider: ID) -> bool:
+        raise NotImplemented
+
+
+class LoginDBI(ABC):
+    """ Login Command Table """
+
+    #
+    #   login command message
+    #
+    @abstractmethod
+    async def get_login_command_message(self, user: ID) -> Tuple[Optional[LoginCommand], Optional[ReliableMessage]]:
+        raise NotImplemented
+
+    @abstractmethod
+    async def save_login_command_message(self, user: ID, content: LoginCommand, msg: ReliableMessage) -> bool:
         raise NotImplemented
 
 
 # noinspection PyAbstractClass
-class SessionDBI(LoginDBI, ProviderDBI, StationDBI, ABC):
+class SessionDBI(ProviderDBI, StationDBI, LoginDBI, ABC):
     """ Session Database """
     pass
