@@ -35,16 +35,17 @@
     Transform and send message
 """
 
-from abc import ABC, abstractmethod
+from abc import ABC
 from typing import Optional, Union, Tuple
 
 from dimsdk import SymmetricKey
 from dimsdk import ID
 from dimsdk import Content, Envelope
-from dimsdk import Command, DocumentCommand
-from dimsdk import InstantMessage, SecureMessage, ReliableMessage
-from dimsdk import EntityDelegate, CipherKeyDelegate
+from dimsdk import Command
+from dimsdk import InstantMessage, ReliableMessage
+from dimsdk import CipherKeyDelegate
 from dimsdk import Messenger, Packer, Processor
+from dimsdk import Compressor
 
 from ..utils import Logging, Converter
 
@@ -53,7 +54,8 @@ from .dbi import MessageDBI
 from .facebook import CommonFacebook
 from .session import Transmitter, Session
 
-from .compat import fix_command
+from .compat import CompatibleOutgoing
+from .compat import CompatibleCompressor
 
 
 class CommonMessenger(Messenger, Transmitter, Logging, ABC):
@@ -65,6 +67,7 @@ class CommonMessenger(Messenger, Transmitter, Logging, ABC):
         self.__database = database
         self.__packer: Optional[Packer] = None
         self.__processor: Optional[Processor] = None
+        self.__compressor = CompatibleCompressor()
 
     @property  # Override
     def packer(self) -> Packer:
@@ -91,32 +94,27 @@ class CommonMessenger(Messenger, Transmitter, Logging, ABC):
         return self.__database
 
     @property  # Override
-    def barrack(self) -> EntityDelegate:
-        return self.__facebook
-
-    @property
     def facebook(self) -> CommonFacebook:
         return self.__facebook
 
+    @property  # Override
+    def compressor(self) -> Compressor:
+        return self.__compressor
+
     @property
     def session(self) -> Session:
         return self.__session
 
-    @abstractmethod
-    def handshake_success(self):
-        """ callback for handshake success """
-        raise NotImplemented
-
     # Override
-    def encrypt_key(self, data: bytes, receiver: ID, msg: InstantMessage) -> Optional[bytes]:
+    async def encrypt_key(self, data: bytes, receiver: ID, msg: InstantMessage) -> Optional[bytes]:
         try:
-            return super().encrypt_key(data=data, receiver=receiver, msg=msg)
+            return await super().encrypt_key(data=data, receiver=receiver, msg=msg)
         except Exception as error:
             # FIXME:
             self.error(msg='failed to encrypt key: %s' % error)
 
     # Override
-    def serialize_key(self, key: Union[dict, SymmetricKey], msg: InstantMessage) -> Optional[bytes]:
+    async def serialize_key(self, key: Union[dict, SymmetricKey], msg: InstantMessage) -> Optional[bytes]:
         # TODO: reuse message key
         #
         # 0. check message key
@@ -124,12 +122,12 @@ class CommonMessenger(Messenger, Transmitter, Logging, ABC):
         digest = key.get('digest')
         if reused is None and digest is None:
             # flags not exist, serialize it directly
-            return super().serialize_key(key=key, msg=msg)
+            return await super().serialize_key(key=key, msg=msg)
         # 1. remove before serializing key
         key.pop('reused', None)
         key.pop('digest', None)
         # 2. serialize key without flags
-        data = super().serialize_key(key=key, msg=msg)
+        data = await super().serialize_key(key=key, msg=msg)
         # 3. put them back after serialized
         if Converter.get_bool(value=reused, default=False):
             key['reused'] = reused
@@ -139,41 +137,33 @@ class CommonMessenger(Messenger, Transmitter, Logging, ABC):
         return data
 
     # Override
-    def serialize_content(self, content: Content, key: SymmetricKey, msg: InstantMessage) -> bytes:
-        if isinstance(content, Command):
-            content = fix_command(content=content)
-        return super().serialize_content(content=content, key=key, msg=msg)
-
-    # Override
-    def deserialize_content(self, data: bytes, key: SymmetricKey, msg: SecureMessage) -> Optional[Content]:
-        content = super().deserialize_content(data=data, key=key, msg=msg)
-        if isinstance(content, Command):
-            content = fix_command(content=content)
-        return content
+    async def serialize_content(self, content: Content, key: SymmetricKey, msg: InstantMessage) -> bytes:
+        CompatibleOutgoing.fix_content(content=content)
+        return await super().serialize_content(content=content, key=key, msg=msg)
 
     #
     #   Interfaces for Transmitting Message
     #
 
     # Override
-    def send_content(self, sender: Optional[ID], receiver: ID, content: Content,
-                     priority: int = 0) -> Tuple[InstantMessage, Optional[ReliableMessage]]:
+    async def send_content(self,  content: Content, sender: Optional[ID], receiver: ID,
+                           priority: int = 0) -> Tuple[InstantMessage, Optional[ReliableMessage]]:
         """ Send message content with priority """
         if sender is None:
-            current = self.facebook.current_user
+            current = await self.facebook.current_user
             assert current is not None, 'current user not set'
             sender = current.identifier
         env = Envelope.create(sender=sender, receiver=receiver)
         i_msg = InstantMessage.create(head=env, body=content)
-        r_msg = self.send_instant_message(msg=i_msg, priority=priority)
+        r_msg = await self.send_instant_message(msg=i_msg, priority=priority)
         return i_msg, r_msg
 
     # private
-    def _attach_visa_time(self, sender: ID, msg: InstantMessage) -> bool:
-        if isinstance(msg.content, DocumentCommand):
+    async def _attach_visa_time(self, sender: ID, msg: InstantMessage) -> bool:
+        if isinstance(msg.content, Command):
             # no need to attach times for command
-            return True
-        doc = self.facebook.visa(identifier=sender)
+            return False
+        doc = await self.facebook.get_visa(sender)
         if doc is None:
             self.error(msg='failed to get visa document for sender: %s' % sender)
             return False
@@ -187,53 +177,59 @@ class CommonMessenger(Messenger, Transmitter, Logging, ABC):
         return True
 
     # Override
-    def send_instant_message(self, msg: InstantMessage, priority: int = 0) -> Optional[ReliableMessage]:
+    async def send_instant_message(self, msg: InstantMessage, priority: int = 0) -> Optional[ReliableMessage]:
         """ send instant message with priority """
         sender = msg.sender
-        # 0. check cycled message
+        #
+        #  0. check cycled message
+        #
         if sender == msg.receiver:
-            self.warning(msg='drop cycled message: %s => %s, %s' % (sender, msg.receiver, msg.group))
+            self.warning(msg='cycled message: %s => %s, %s' % (sender, msg.receiver, msg.group))
             # return None
         else:
-            self.debug(msg='send instant message message (type=%d): %s => %s, %s'
+            self.debug(msg='send instant message message (type=%s): %s => %s, %s'
                            % (msg.content.type, sender, msg.receiver, msg.group))
             # attach sender's document times
             # for the receiver to check whether user info synchronized
-            ok = self._attach_visa_time(sender=sender, msg=msg)
-            if not ok:
+            ok = await self._attach_visa_time(sender=sender, msg=msg)
+            if ok or isinstance(msg.content, Command):
+                pass
+            else:
                 self.warning(msg='failed to attach document time: %s => %s' % (sender, msg.content))
         #
         #  1. encrypt message
         #
-        s_msg = self.encrypt_message(msg=msg)
+        s_msg = await self.encrypt_message(msg=msg)
         if s_msg is None:
             # public key not found?
+            self.warning(msg='failed to encrypt message: %s => %s, %s' % (sender, msg.receiver, msg.group))
             return None
         #
         #  2. sign message
         #
-        r_msg = self.sign_message(msg=s_msg)
+        r_msg = await self.sign_message(msg=s_msg)
         if r_msg is None:
             # TODO: set msg.state = error
             raise AssertionError('failed to sign message: %s' % s_msg)
         #
         #  3. send message
         #
-        if self.send_reliable_message(msg=r_msg, priority=priority):
+        if await self.send_reliable_message(msg=r_msg, priority=priority):
             return r_msg
         # failed
+        self.error(msg='failed to send message: %s => %s, %s' % (sender, msg.receiver, msg.group))
 
     # Override
-    def send_reliable_message(self, msg: ReliableMessage, priority: int = 0) -> bool:
+    async def send_reliable_message(self, msg: ReliableMessage, priority: int = 0) -> bool:
         """ send reliable message with priority """
-        # # 0. check cycled message
-        # if msg.sender == msg.receiver:
-        #     self.warning(msg='drop cycled message: %s => %s, %s' % (msg.sender, msg.receiver, msg.group))
-        #     return False
+        # 0. check cycled message
+        if msg.sender == msg.receiver:
+            self.warning(msg='cycled message: %s => %s, %s' % (msg.sender, msg.receiver, msg.group))
+            # return False
         # 1. serialize message
-        data = self.serialize_message(msg=msg)
+        data = await self.serialize_message(msg=msg)
         assert data is not None, 'failed to serialize message: %s' % msg
         # 2. call gate keeper to send the message data package
         #    put message package into the waiting queue of current session
         session = self.session
-        return session.queue_message_package(msg=msg, data=data, priority=priority)
+        return await session.queue_message_package(msg=msg, data=data, priority=priority)
