@@ -23,104 +23,77 @@
 # SOFTWARE.
 # ==============================================================================
 
-import threading
 from abc import ABC
 from typing import Optional, List, Dict
 
 from dimsdk import EncryptKey
 from dimsdk import ID
-from dimsdk import ReceiptCommand, DocumentCommand
 from dimsdk import InstantMessage, SecureMessage, ReliableMessage
-from dimsdk import MessagePacker, MessageHelper
-from dimsdk import Facebook, Messenger
+from dimsdk import MessagePacker
+from dimsdk import MessageUtils
+from dimsdk import Compressor
 
 from ..utils import Logging
 
-from .compat import fix_meta_attachment
-from .compat import fix_receipt_command
-from .compat import fix_document_command
+from .facebook import CommonFacebook
+from .messenger import CommonMessenger
+from .compat import Compatible
+
+from .queue import SuspendedMessageQueue
 
 
 class CommonMessagePacker(MessagePacker, Logging, ABC):
 
-    def __init__(self, facebook: Facebook, messenger: Messenger):
+    def __init__(self, facebook: CommonFacebook, messenger: CommonMessenger):
         super().__init__(facebook=facebook, messenger=messenger)
-        # suspended messages
-        self.__suspend_lock = threading.Lock()
-        self.__incoming_messages: List[ReliableMessage] = []
-        self.__outgoing_messages: List[InstantMessage] = []
+        self.__queue = SuspendedMessageQueue()
 
-    #
-    #   Suspending
-    #
+    @property  # Override
+    def messenger(self) -> Optional[CommonMessenger]:
+        transceiver = super().messenger
+        assert isinstance(transceiver, CommonMessenger), 'transceiver error: %s' % transceiver
+        return transceiver
+
+    @property  # Override
+    def compressor(self) -> Compressor:
+        return self.messenger.compressor
 
     def suspend_reliable_message(self, msg: ReliableMessage, error: Dict):
-        """
-        Add income message in a queue for waiting sender's visa
-
-        :param msg:   incoming message
-        :param error: error info
-        """
-        self.warning(msg='suspend message: %s -> %s, %s' % (msg.sender, msg.receiver, error))
-        msg['error'] = error
-        with self.__suspend_lock:
-            if len(self.__incoming_messages) > 32:
-                self.__incoming_messages.pop(0)
-            self.__incoming_messages.append(msg)
+        self.__queue.suspend_reliable_message(msg=msg, error=error)
 
     def suspend_instant_message(self, msg: InstantMessage, error: Dict):
-        """
-        Add outgo message in a queue for waiting receiver's visa
-
-        :param msg:   outgo message
-        :param error: error info
-        """
-        self.warning(msg='suspend message: %s -> %s, %s' % (msg.sender, msg.receiver, error))
-        msg['error'] = error
-        with self.__suspend_lock:
-            if len(self.__outgoing_messages) > 32:
-                self.__outgoing_messages.pop(0)
-            self.__outgoing_messages.append(msg)
+        self.__queue.suspend_instant_message(msg=msg, error=error)
 
     def resume_reliable_messages(self) -> List[ReliableMessage]:
-        with self.__suspend_lock:
-            messages = self.__incoming_messages
-            self.__incoming_messages = []
-            return messages
+        return self.__queue.resume_reliable_messages()
 
     def resume_instant_messages(self) -> List[InstantMessage]:
-        with self.__suspend_lock:
-            messages = self.__outgoing_messages
-            self.__outgoing_messages = []
-            return messages
+        return self.__queue.resume_instant_messages()
 
     #
     #   Checking
     #
 
     # protected
-    def _visa_key(self, user: ID) -> Optional[EncryptKey]:
+    async def _visa_key(self, user: ID) -> Optional[EncryptKey]:
         """ for checking whether user's ready """
-        return self.facebook.public_key_for_encryption(identifier=user)
+        db = self.facebook
+        return await db.public_key_for_encryption(identifier=user)
 
     # protected
-    def _members(self, group: ID) -> List[ID]:
-        """ for checking whether group's ready """
-        return self.facebook.members(identifier=group)
-
-    # protected
-    def _check_sender(self, msg: ReliableMessage) -> bool:
+    async def _check_sender(self, msg: ReliableMessage) -> bool:
         """ Check sender before verifying received message """
         sender = msg.sender
         assert sender.is_user, 'sender error: %s' % sender
         # check sender's meta & document
-        visa = MessageHelper.get_visa(msg=msg)
+        visa = MessageUtils.get_visa(msg=msg)
         if visa is not None:
             # first handshake?
-            assert visa.identifier == sender, 'visa ID not match: %s => %s' % (sender, visa)
+            matched = visa.identifier == sender
+            assert matched, 'visa ID not match: %s => %s' % (sender, visa)
             # assert Meta.match_id(meta=msg.meta, identifier=sender), 'meta error: %s' % msg
-            return visa.identifier == sender
-        elif self._visa_key(user=sender) is not None:
+            return matched
+        elif await self._visa_key(user=sender) is not None:
             # sender is OK
             return True
         # sender not ready, suspend message for waiting document
@@ -132,7 +105,7 @@ class CommonMessagePacker(MessagePacker, Logging, ABC):
         return False
 
     # protected
-    def _check_receiver(self, msg: InstantMessage) -> bool:
+    async def _check_receiver(self, msg: InstantMessage) -> bool:
         """ Check receiver before encrypting message """
         receiver = msg.receiver
         if receiver.is_broadcast:
@@ -145,7 +118,7 @@ class CommonMessagePacker(MessagePacker, Logging, ABC):
             #         that should be sent to a group bot first,
             #         and the bot will separate it for all members.
             return False
-        elif self._visa_key(user=receiver) is not None:
+        elif await self._visa_key(user=receiver) is not None:
             # receiver is OK
             return True
         # receiver not ready, suspend message for waiting document
@@ -161,63 +134,52 @@ class CommonMessagePacker(MessagePacker, Logging, ABC):
     #
 
     # Override
-    def encrypt_message(self, msg: InstantMessage) -> Optional[SecureMessage]:
+    async def encrypt_message(self, msg: InstantMessage) -> Optional[SecureMessage]:
         # 1. check contact info
         # 2. check group members info
-        if not self._check_receiver(msg=msg):
-            # receiver not ready
+        if await self._check_receiver(msg=msg):
+            # receiver is ready
+            pass
+        else:
             self.warning(msg='receiver not ready: %s' % msg.receiver)
             return None
-        content = msg.content
-        if isinstance(content, ReceiptCommand):
-            # compatible with v1.0
-            fix_receipt_command(content=content)
-        return super().encrypt_message(msg=msg)
+        return await super().encrypt_message(msg=msg)
 
     # Override
-    def decrypt_message(self, msg: SecureMessage) -> Optional[InstantMessage]:
-        i_msg = super().decrypt_message(msg=msg)
-        if i_msg is not None:
-            content = i_msg.content
-            if isinstance(content, ReceiptCommand):
-                # compatible with v1.0
-                fix_receipt_command(content=content)
-            elif isinstance(content, DocumentCommand):
-                # compatible with v1.0
-                fix_document_command(content=content)
-        return i_msg
-
-    # Override
-    def verify_message(self, msg: ReliableMessage) -> Optional[SecureMessage]:
+    async def verify_message(self, msg: ReliableMessage) -> Optional[SecureMessage]:
         # 1. check receiver/group with local user
         # 2. check sender's meta
-        if not self._check_sender(msg=msg):
-            # sender not ready
+        if await self._check_sender(msg=msg):
+            # sender is ready
+            pass
+        else:
             self.warning(msg='sender not ready: %s' % msg.sender)
             return None
-        return super().verify_message(msg=msg)
+        return await super().verify_message(msg=msg)
 
     # Override
-    def sign_message(self, msg: SecureMessage) -> ReliableMessage:
+    async def sign_message(self, msg: SecureMessage) -> ReliableMessage:
         if isinstance(msg, ReliableMessage):
             # already signed
             return msg
-        return super().sign_message(msg=msg)
+        return await super().sign_message(msg=msg)
 
     # Override
-    def deserialize_message(self, data: bytes) -> Optional[ReliableMessage]:
-        if data is None or len(data) < 2:
+    async def deserialize_message(self, data: bytes) -> Optional[ReliableMessage]:
+        if data is None or len(data) <= 4:
             # message data error
             return None
         # elif not (data.startswith(b'{') and data.endswith(b'}')):
         #     # only support JsON format now
         #     return None
-        msg = super().deserialize_message(data=data)
+        msg = await super().deserialize_message(data=data)
         if msg is not None:
-            fix_meta_attachment(msg=msg)
+            Compatible.fix_meta_attachment(msg=msg)
+            Compatible.fix_visa_attachment(msg=msg)
         return msg
 
     # Override
-    def serialize_message(self, msg: ReliableMessage) -> bytes:
-        fix_meta_attachment(msg=msg)
-        return super().serialize_message(msg=msg)
+    async def serialize_message(self, msg: ReliableMessage) -> bytes:
+        Compatible.fix_meta_attachment(msg=msg)
+        Compatible.fix_visa_attachment(msg=msg)
+        return await super().serialize_message(msg=msg)
