@@ -23,49 +23,43 @@
 # SOFTWARE.
 # ==============================================================================
 
-from dimsdk import Command
-from dimsdk import CommandFactoryBuilder
-from dimsdk import register_all_factories as register_core_factories
-from dimplugins import register_plugins
+from .version import MetaVersion
+from .password import Password
+from .utils import BroadcastUtils
+
+from .ans import AnsCommand
 
 from .handshake import HandshakeCommand, HandshakeState
 from .login import LoginCommand
-from .report import ReportCommand
-from .ans import AnsCommand
 
 from .mute import MuteCommand
 from .block import BlockCommand
 
+from .report import ReportCommand
 
-def register_all_factories():
-    # Register core factories
-    register_core_factories()
-
-    # Handshake
-    Command.register(cmd=HandshakeCommand.HANDSHAKE, factory=CommandFactoryBuilder(command_class=HandshakeCommand))
-    # Login
-    Command.register(cmd=LoginCommand.LOGIN, factory=CommandFactoryBuilder(command_class=LoginCommand))
-    # Report
-    Command.register(cmd=ReportCommand.REPORT, factory=CommandFactoryBuilder(command_class=ReportCommand))
-    # ANS
-    Command.register(cmd=AnsCommand.ANS, factory=CommandFactoryBuilder(command_class=AnsCommand))
-
-    # Mute
-    Command.register(cmd=MuteCommand.MUTE, factory=CommandFactoryBuilder(command_class=MuteCommand))
-    # Block
-    Command.register(cmd=BlockCommand.BLOCK, factory=CommandFactoryBuilder(command_class=BlockCommand))
+from .customized import CustomizedContent, AppCustomizedContent
 
 
 __all__ = [
 
+    'MetaVersion',
+    'Password',
+    'BroadcastUtils',
+
+    #
+    #   Commands
+    #
+
+    'AnsCommand',
+
     'HandshakeCommand', 'HandshakeState',
     'LoginCommand',
-    'ReportCommand',
-    'AnsCommand',
 
     'BlockCommand',
     'MuteCommand',
 
-    'register_all_factories',
-    'register_plugins',
+    'ReportCommand',
+
+    'CustomizedContent', 'AppCustomizedContent',
+
 ]
