@@ -29,12 +29,12 @@
 
         +--------------+
         |   Facebook   |
-        +--------------+         DockerDelegate      ConnectionDelegate
+        +--------------+         PorterDelegate      ConnectionDelegate
               :  AccountDB      +--------------+        +----------+
               :                 |  GateKeeper  | . . . >|   Gate   |
         +--------------+        +--------------+        +----------+
         |   Messenger  | . . . >|   Session    |            Hub (Connection)
-        +--------------+        +--------------+            Docker
+        +--------------+        +--------------+            Porter
                  MessageDB               SessionDB
                  Filter
                  Dispatcher
@@ -42,27 +42,28 @@
 
 from startrek import Hub, Channel
 from startrek import Connection, ConnectionDelegate, ConnectionState
-from startrek import BaseChannel, ChannelReader, ChannelWriter
+from startrek import BaseChannel
 from startrek import BaseHub, BaseConnection, ActiveConnection
 
 from startrek import Ship, Arrival, Departure, DeparturePriority
-from startrek import Docker, DockerStatus, DockerDelegate, Gate
+from startrek import Porter, PorterStatus, PorterDelegate, Gate
 from startrek import ArrivalShip, ArrivalHall, DepartureShip, DepartureHall
-from startrek import Dock, LockedDock, StarDocker, StarGate
+from startrek import Dock, LockedDock, StarPorter, StarGate
 
-from tcp import PlainArrival, PlainDeparture, PlainDocker
+from tcp import PlainArrival, PlainDeparture, PlainPorter
 from tcp import StreamChannel, StreamHub
 from tcp import ServerHub as TCPServerHub, ClientHub as TCPClientHub
 
-from udp import PackageArrival, PackageDeparture, PackageDocker
+from udp import PackageArrival, PackageDeparture, PackagePorter
 from udp import PacketChannel, PacketHub
 from udp import ServerHub as UDPServerHub, ClientHub as UDPClientHub
 
 from .protocol import WebSocket, NetMsg, NetMsgHead, NetMsgSeq
 
-from .ws import WSArrival, WSDeparture, WSDocker
-from .mars import MarsStreamArrival, MarsStreamDeparture, MarsStreamDocker
-from .mtp import MTPStreamArrival, MTPStreamDeparture, MTPStreamDocker
+from .ws import WSArrival, WSDeparture, WSPorter
+from .mars import MarsStreamArrival, MarsStreamDeparture, MarsStreamPorter
+from .mtp import MTPStreamArrival, MTPStreamDeparture, MTPStreamPorter
+from .flexible import FlexiblePorter
 from .gate import CommonGate, TCPServerGate, TCPClientGate, UDPServerGate, UDPClientGate
 # from .gatekeeper import GateKeeper
 from .queue import MessageWrapper, MessageQueue
@@ -76,24 +77,24 @@ __all__ = [
     #
     'Hub', 'Channel',
     'Connection', 'ConnectionDelegate', 'ConnectionState',
-    'BaseChannel', 'ChannelReader', 'ChannelWriter',
+    'BaseChannel',
     'BaseHub', 'BaseConnection', 'ActiveConnection',
 
     'Ship', 'Arrival', 'Departure', 'DeparturePriority',
-    'Docker', 'DockerStatus', 'DockerDelegate', 'Gate',
+    'Porter', 'PorterStatus', 'PorterDelegate', 'Gate',
     'ArrivalShip', 'ArrivalHall', 'DepartureShip', 'DepartureHall',
-    'Dock', 'LockedDock', 'StarDocker', 'StarGate',
+    'Dock', 'LockedDock', 'StarPorter', 'StarGate',
 
     #
     #   TCP
     #
-    'PlainArrival', 'PlainDeparture', 'PlainDocker',
+    'PlainArrival', 'PlainDeparture', 'PlainPorter',
     'StreamChannel', 'StreamHub', 'TCPServerHub', 'TCPClientHub',
 
     #
     #   UDP
     #
-    'PackageArrival', 'PackageDeparture', 'PackageDocker',
+    'PackageArrival', 'PackageDeparture', 'PackagePorter',
     'PacketChannel', 'PacketHub', 'UDPServerHub', 'UDPClientHub',
 
     #
@@ -104,11 +105,13 @@ __all__ = [
     #
     #   Network
     #
-    'WSArrival', 'WSDeparture', 'WSDocker',
-    'MarsStreamArrival', 'MarsStreamDeparture', 'MarsStreamDocker',
-    'MTPStreamArrival', 'MTPStreamDeparture', 'MTPStreamDocker',
+    'WSArrival', 'WSDeparture', 'WSPorter',
+    'MarsStreamArrival', 'MarsStreamDeparture', 'MarsStreamPorter',
+    'MTPStreamArrival', 'MTPStreamDeparture', 'MTPStreamPorter',
+    'FlexiblePorter',
     'CommonGate', 'TCPServerGate', 'TCPClientGate', 'UDPServerGate', 'UDPClientGate',
     # 'GateKeeper',
     'MessageWrapper', 'MessageQueue',
     'BaseSession',
+
 ]
