@@ -31,7 +31,7 @@
 import socket
 import time
 import traceback
-from typing import Optional, Tuple
+from typing import Optional
 
 from dimsdk import ReliableMessage
 
@@ -40,14 +40,14 @@ from startrek import Channel, Hub
 from startrek import BaseChannel
 from startrek import Connection, ConnectionDelegate, BaseConnection
 from startrek import Arrival, Departure
-from startrek import Docker, DockerStatus, DockerDelegate
+from startrek import Porter, PorterStatus, PorterDelegate
 
 from tcp import StreamChannel
 from tcp import ServerHub, ClientHub
 
 from ..utils import get_remote_address, get_local_address
 from ..utils import get_msg_info
-from ..utils import Runner, Logging
+from ..utils import Runner, Log, Logging
 
 from .protocol import DeparturePacker
 
@@ -130,7 +130,7 @@ def reset_send_buffer_size(conn: Connection = None, sock: socket.socket = None)
             return False
         channel = conn.channel
         if not isinstance(channel, BaseChannel):
-            print('[SOCKET] channel error: %s, %s' % (channel, conn))
+            print('[SOCKET] channel error: %s, %s' % (channel, conn.remote_address))
             return False
         sock = channel.sock
         if sock is None:
@@ -146,12 +146,21 @@ def reset_send_buffer_size(conn: Connection = None, sock: socket.socket = None)
         print('[SOCKET] send buffer size: %d, %s' % (size, conn))
 
 
-class GateKeeper(Runner, DockerDelegate, Logging):
+async def _client_connect(hub: Hub, address):
+    conn = await hub.connect(remote=address)
+    if conn is None:
+        Log.error(msg='failed to connect to remote address: %s' % str(address))
+    else:
+        Log.info(msg='connected to remote address: %s, %s' % (address, conn))
+        reset_send_buffer_size(conn=conn)
+
+
+class GateKeeper(Runner, PorterDelegate, Logging):
     """ Keep a gate to remote address """
 
     SEND_BUFFER_SIZE = 64 * 1024  # 64 KB
 
-    def __init__(self, remote: Tuple[str, int], sock: Optional[socket.socket]):
+    def __init__(self, remote: SocketAddress, sock: Optional[socket.socket]):
         super().__init__(interval=Runner.INTERVAL_SLOW)
         self.__remote = remote
         self.__queue = MessageQueue()
@@ -159,7 +168,7 @@ class GateKeeper(Runner, DockerDelegate, Logging):
         self.__last_active = 0  # last update time
         self.__gate = self._create_gate(remote=remote, sock=sock)
 
-    def _create_gate(self, remote: Tuple[str, int], sock: Optional[socket.socket]) -> CommonGate:
+    def _create_gate(self, remote: SocketAddress, sock: Optional[socket.socket]) -> CommonGate:
         if sock is None:
             gate = TCPClientGate(delegate=self)
         else:
@@ -168,14 +177,14 @@ class GateKeeper(Runner, DockerDelegate, Logging):
         return gate
 
     # noinspection PyMethodMayBeStatic
-    def _create_hub(self, delegate: ConnectionDelegate, address: Tuple[str, int], sock: Optional[socket.socket]) -> Hub:
+    def _create_hub(self, delegate: ConnectionDelegate, address: SocketAddress, sock: Optional[socket.socket]) -> Hub:
         if sock is None:
             # client
             assert address is not None, 'remote address empty'
             hub = StreamClientHub(delegate=delegate)
-            conn = hub.connect(remote=address)
-            reset_send_buffer_size(conn=conn)
-            # TODO: reset send buffer size
+            coro = _client_connect(hub=hub, address=address)
+            Runner.async_task(coro=coro)
+            self.info(msg='client hub created: %s' % str(address))
         else:
             # server
             sock.setblocking(False)
@@ -183,13 +192,14 @@ class GateKeeper(Runner, DockerDelegate, Logging):
             if address is None:
                 address = get_remote_address(sock=sock)
             channel = StreamChannel(remote=address, local=get_local_address(sock=sock))
-            channel.set_socket(sock=sock)
+            coro = channel.set_socket(sock=sock)
+            Runner.async_task(coro=coro)
             hub = StreamServerHub(delegate=delegate)
             hub.put_channel(channel=channel)
         return hub
 
     @property
-    def remote_address(self) -> Tuple[str, int]:
+    def remote_address(self) -> SocketAddress:
         return self.__remote
 
     @property
@@ -210,35 +220,15 @@ class GateKeeper(Runner, DockerDelegate, Logging):
             self.__last_active = when
             return True
 
-    # @property  # Override
-    # def running(self) -> bool:
-    #     if super().running:
-    #         return self.gate.running
-    #
-    # # Override
-    # def stop(self):
-    #     super().stop()
-    #     self.gate.stop()
-    #
-    # # Override
-    # def setup(self):
-    #     super().setup()
-    #     self.gate.start()
-    #
-    # # Override
-    # def finish(self):
-    #     self.gate.stop()
-    #     super().finish()
-
     # Override
-    def process(self) -> bool:
+    async def process(self) -> bool:
         gate = self.gate
         hub = gate.hub
         # from tcp import Hub
         # assert isinstance(hub, Hub), 'hub error: %s' % hub
         try:
-            incoming = hub.process()
-            outgoing = gate.process()
+            incoming = await hub.process()
+            outgoing = await gate.process()
             if incoming or outgoing:
                 # processed income/outgo packages
                 return True
@@ -263,15 +253,15 @@ class GateKeeper(Runner, DockerDelegate, Logging):
             # msg sent?
             return True
         # try to push
-        ok = gate.send_ship(ship=wrapper, remote=self.remote_address, local=None)
+        ok = await gate.send_ship(ship=wrapper, remote=self.remote_address, local=None)
         if not ok:
             self.error(msg='gate error, failed to send data')
         return ok
 
-    def _docker_pack(self, payload: bytes, priority: int = 0) -> Departure:
-        docker = self.gate.fetch_docker([], remote=self.remote_address, local=None)
+    async def _porter_pack(self, payload: bytes, priority: int = 0) -> Optional[Departure]:
+        docker = await self.gate.fetch_porter(remote=self.remote_address, local=None)
         assert isinstance(docker, DeparturePacker), 'departure packer error: %s' % docker
-        return docker.pack(payload=payload, priority=priority)
+        return docker.pack(payload=payload, priority=priority, needs_respond=True)
 
     def _queue_append(self, msg: ReliableMessage, ship: Departure) -> bool:
         return self.__queue.append(msg=msg, ship=ship)
@@ -281,25 +271,25 @@ class GateKeeper(Runner, DockerDelegate, Logging):
     #
 
     # Override
-    def docker_status_changed(self, previous: DockerStatus, current: DockerStatus, docker: Docker):
-        self.info(msg='docker status changed: %s -> %s, %s' % (previous, current, docker))
+    async def porter_status_changed(self, previous: PorterStatus, current: PorterStatus, porter: Porter):
+        self.info(msg='docker status changed: %s -> %s, %s' % (previous, current, porter))
 
     # Override
-    def docker_received(self, ship: Arrival, docker: Docker):
-        self.debug(msg='docker received a ship: %s, %s' % (ship, docker))
+    async def porter_received(self, ship: Arrival, porter: Porter):
+        self.debug(msg='docker received a ship: %s, %s' % (ship, porter))
 
     # Override
-    def docker_sent(self, ship: Departure, docker: Docker):
+    async def porter_sent(self, ship: Departure, porter: Porter):
         # TODO: remove sent message from local cache
         pass
 
     # Override
-    def docker_failed(self, error: IOError, ship: Departure, docker: Docker):
-        self.error(msg='docker failed to send ship: %s, %s' % (error, docker))
+    async def porter_failed(self, error: IOError, ship: Departure, porter: Porter):
+        self.error(msg='docker failed to send ship: %s, %s' % (error, porter))
 
     # Override
-    def docker_error(self, error: IOError, ship: Departure, docker: Docker):
-        self.error(msg='docker error while sending ship: %s, %s' % (error, docker))
+    async def porter_error(self, error: IOError, ship: Departure, porter: Porter):
+        self.error(msg='docker error while sending ship: %s, %s' % (error, porter))
         if isinstance(ship, MessageWrapper):
             msg = ship.msg
             if msg is not None:
