@@ -23,13 +23,16 @@
 # SOFTWARE.
 # ==============================================================================
 
+import threading
 from typing import Optional
 
 from dimsdk import DateTime
 from dimsdk import ID, SymmetricKey
-from dimplugins import PlainKey
+from dimsdk import SymmetricAlgorithms
 
-from ..utils import CacheManager
+from ..utils import Config
+from ..utils import SharedCacheManager
+from ..common import Password
 from ..common import CipherKeyDBI
 
 
@@ -39,10 +42,11 @@ class CipherKeyTable(CipherKeyDBI):
     CACHE_EXPIRES = 3600*24*7  # seconds
 
     # noinspection PyUnusedLocal
-    def __init__(self, root: str = None, public: str = None, private: str = None):
+    def __init__(self, config: Config):
         super().__init__()
-        man = CacheManager()
-        self.__keys_cache = man.get_pool(name='cipher_keys')  # (ID, ID) => SymmetricKey
+        man = SharedCacheManager()
+        self._cache = man.get_pool(name='cipher_keys')  # (ID, ID) => SymmetricKey
+        self._lock = threading.Lock()
 
     # noinspection PyMethodMayBeStatic
     def show_info(self):
@@ -53,26 +57,45 @@ class CipherKeyTable(CipherKeyDBI):
     #
 
     # Override
-    def cipher_key(self, sender: ID, receiver: ID, generate: bool = False) -> Optional[SymmetricKey]:
+    async def get_cipher_key(self, sender: ID, receiver: ID, generate: bool = False) -> Optional[SymmetricKey]:
         if receiver.is_broadcast:
-            return PlainKey()
+            return Password.kPlainKey
         now = DateTime.now()
         direction = (sender, receiver)
-        key, _ = self.__keys_cache.fetch(key=direction, now=now)
-        if key is None and generate:
-            # generate and cache it
-            key = SymmetricKey.generate(algorithm=SymmetricKey.AES)
-            assert key is not None, 'failed to generate symmetric key'
-            self.__keys_cache.update(key=direction, value=key, life_span=self.CACHE_EXPIRES, now=now)
-        return key
+        cache_pool = self._cache
+        #
+        #  1. check memory cache
+        #
+        pwd, _ = cache_pool.fetch(key=direction, now=now)
+        if pwd is not None:
+            # got it from cache
+            return pwd
+        #
+        #  2. lock for querying
+        #
+        with self._lock:
+            # locked, check again to make sure the cache not exists.
+            # (maybe the cache was updated by other threads while waiting the lock)
+            pwd, _ = cache_pool.fetch(key=direction, now=now)
+            if pwd is None and generate:
+                # generate and cache it
+                pwd = SymmetricKey.generate(algorithm=SymmetricAlgorithms.AES)
+                assert pwd is not None, 'failed to generate symmetric key'
+                cache_pool.update(key=direction, value=pwd, life_span=self.CACHE_EXPIRES, now=now)
+        #
+        #  3. OK, return cached value
+        #
+        return pwd
 
     # Override
-    def cache_cipher_key(self, key: SymmetricKey, sender: ID, receiver: ID):
+    async def cache_cipher_key(self, key: SymmetricKey, sender: ID, receiver: ID):
         if receiver.is_broadcast:
             # no need to store cipher key for broadcast message
             return False
         now = DateTime.now()
         direction = (sender, receiver)
-        # 1. store into memory cache
-        self.__keys_cache.update(key=direction, value=key, life_span=self.CACHE_EXPIRES, now=now)
+        cache_pool = self._cache
+        with self._lock:
+            # store into memory cache
+            cache_pool.update(key=direction, value=key, life_span=self.CACHE_EXPIRES, now=now)
         return True
