@@ -51,7 +51,7 @@ class HandshakeCommandProcessor(BaseCommandProcessor):
         return transceiver
 
     # Override
-    def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
+    async def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
         assert isinstance(content, HandshakeCommand), 'handshake command error: %s' % content
         title = content.title
         if title in ['DIM?', 'DIM!']:
@@ -72,21 +72,21 @@ class HandshakeCommandProcessor(BaseCommandProcessor):
         sender = r_msg.sender
         if sess_id is not None:
             assert sess_id == sender, 'sender error: %s, %s' % (sender, sess_id)
-        if session.key == content.session:
+        if session.session_key == content.session:
             # session key match
-            Log.info(msg='handshake accepted: %s, session: %s' % (sender, session.key))
+            Log.info(msg='handshake accepted: %s, session: %s' % (sender, session.session_key))
             # verified success
-            handshake_accepted(identifier=sender, when=content.time, session=session, messenger=messenger)
-            res = HandshakeCommand.success(session=session.key)
+            await handshake_accepted(identifier=sender, when=content.time, session=session, messenger=messenger)
+            res = HandshakeCommand.success(session=session.session_key)
         else:
             # session key not match
             # ask client to sign it with the new session key
-            res = HandshakeCommand.again(session=session.key)
+            res = HandshakeCommand.again(session=session.session_key)
         res['remote_address'] = session.remote_address
         return [res]
 
 
-def handshake_accepted(identifier: ID, when: Optional[DateTime], session: Session, messenger: CommonMessenger):
+async def handshake_accepted(identifier: ID, when: Optional[DateTime], session: Session, messenger: CommonMessenger):
     from ..session_center import SessionCenter
     center = SessionCenter()
     # 1. update session ID
@@ -94,4 +94,6 @@ def handshake_accepted(identifier: ID, when: Optional[DateTime], session: Sessio
     # 2. update session flag
     session.set_active(active=True, when=when)
     # 3. callback
-    messenger.handshake_success()
+    from ..messenger import ServerMessenger
+    assert isinstance(messenger, ServerMessenger)
+    await messenger.handshake_success()
