@@ -30,7 +30,7 @@
     login protocol
 """
 
-from typing import List
+from typing import List, Dict
 
 from dimsdk import ID
 from dimsdk import ReliableMessage
@@ -58,19 +58,26 @@ class LoginCommandProcessor(BaseCommandProcessor, Logging):
         return transceiver
 
     # Override
-    def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
+    async def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
         assert isinstance(content, LoginCommand), 'command error: %s' % content
         sender = content.identifier
-        # 1. store login command
+        # 1. check roaming station
+        station = content.station
+        if not isinstance(station, Dict):
+            self.error(msg='login command error: %s -> %s' % (sender, content))
+            self.error(msg='login command error: %s -> %s' % (sender, r_msg))
+            return []
+        # 2. store login command
         session = self.messenger.session
         db = session.database
-        if not db.save_login_command_message(user=sender, content=content, msg=r_msg):
+        if not await db.save_login_command_message(user=sender, content=content, msg=r_msg):
             self.error(msg='login command error/expired: %s' % content)
             return []
-        # 2. check roaming station
-        current = self.facebook.current_user
-        station = content.station
-        roaming = ID.parse(identifier=station.get('ID'))
+        current = await self.facebook.current_user
+        sid = station.get('did')
+        if sid is None:
+            sid = station.get('ID')
+        roaming = ID.parse(identifier=sid)
         # assert isinstance(roaming, ID), 'login command error: %s' % content
         if not isinstance(roaming, ID):
             self.warning(msg='station ID not found: %s' % station)
@@ -90,9 +97,9 @@ class LoginCommandProcessor(BaseCommandProcessor, Logging):
         self.info(msg='user login: %s -> %s' % (sender, roaming))
         text = 'Login received.'
         return self._respond_receipt(text=text, content=content, envelope=r_msg.envelope, extra={
-            'template': 'Login command received: ${ID}.',
+            'template': 'Login command received: ${did}.',
             'replacements': {
-                'ID': str(sender),
+                'did': str(sender),
             }
         })
 
