@@ -51,7 +51,7 @@ class ReportCommandProcessor(BaseCommandProcessor, Logging):
         return messenger.session
 
     # Override
-    def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
+    async def process_content(self, content: Content, r_msg: ReliableMessage) -> List[Content]:
         assert isinstance(content, ReportCommand), 'report command error: %s' % content
         # check session sender
         session = self.session
@@ -68,9 +68,9 @@ class ReportCommandProcessor(BaseCommandProcessor, Logging):
             session.set_active(active=True, when=content.time)
             text = 'Online received.'
             return self._respond_receipt(text=text, content=content, envelope=r_msg.envelope, extra={
-                'template': 'Online command received: ${ID}.',
+                'template': 'Online command received: ${did}.',
                 'replacements': {
-                    'ID': str(sender),
+                    'did': str(sender),
                 }
             })
         elif title == ReportCommand.OFFLINE:
