@@ -34,9 +34,7 @@ import traceback
 from socketserver import StreamRequestHandler
 
 from ..utils import Logging, Runner
-
 from ..server import ServerSession, SessionCenter
-from ..server import ServerMessenger
 
 from .shared import GlobalVariable
 from .shared import create_messenger
@@ -44,46 +42,6 @@ from .shared import create_messenger
 
 class RequestHandler(StreamRequestHandler, Logging):
 
-    def __init__(self, request, client_address, server):
-        shared = GlobalVariable()
-        session = ServerSession(remote=client_address, sock=request, database=shared.sdb)
-        self.__messenger = create_messenger(facebook=shared.facebook, database=shared.mdb, session=session)
-        # call 'setup()', 'handle()', 'finish()'
-        super().__init__(request=request, client_address=client_address, server=server)
-
-    @property
-    def messenger(self) -> ServerMessenger:
-        return self.__messenger
-
-    # Override
-    def setup(self):
-        super().setup()
-        try:
-            session = self.messenger.session
-            center = SessionCenter()
-            center.add_session(session=session)
-            self.info(msg='client connected: %s' % session)
-            assert isinstance(session, Runner), 'session error: %s' % session
-            session.setup()
-        except Exception as error:
-            self.error(msg='setup request handler error: %s' % error)
-            traceback.print_exc()
-
-    # Override
-    def finish(self):
-        try:
-            session = self.messenger.session
-            self.info(msg='client disconnected: %s' % session)
-            center = SessionCenter()
-            center.remove_session(session=session)
-            assert isinstance(session, Runner), 'session error: %s' % session
-            session.finish()
-            self.__messenger = None
-        except Exception as error:
-            self.error(msg='finish request handler error: %s' % error)
-            traceback.print_exc()
-        super().finish()
-
     """
         DIM Request Handler
     """
@@ -93,10 +51,29 @@ class RequestHandler(StreamRequestHandler, Logging):
         super().handle()
         try:
             self.info(msg='session started: %s' % str(self.client_address))
-            session = self.messenger.session
-            assert isinstance(session, Runner), 'session error: %s' % session
-            session.handle()
+            Runner.sync_run(main=_start_session(handler=self))
             self.info(msg='session finished: %s' % str(self.client_address))
         except Exception as error:
             self.error(msg='request handler error: %s' % error)
             traceback.print_exc()
+
+
+async def _start_session(handler: RequestHandler):
+    client_address = handler.client_address
+    request = handler.request
+    shared = GlobalVariable()
+    session = ServerSession(remote=client_address, sock=request, database=shared.sdb)
+    messenger = create_messenger(facebook=shared.facebook, database=shared.mdb, session=session)
+    center = SessionCenter()
+    # setup
+    center.add_session(session=session)
+    try:
+        # handle
+        # await session.start()
+        await session.run()
+        # await session.stop()
+    finally:
+        # finish
+        center.remove_session(session=session)
+        await Runner.sleep(seconds=2.0)
+    return messenger
